from wanakana.constants import PROLONGED_SOUND_MARK


def is_char_in_range(char: str, start: int, end: int) -> bool:
    if not char:
        return False

    code: int = ord(char)
    return start <= code <= end


def is_char_long_dash(char: str = ""):
    if not char:
        return False

    code: int = ord(char)
    return code == PROLONGED_SOUND_MARK



import re

from wanakana.constants import ROMAJI_RANGES
from wanakana.utils import is_char_in_range


def is_romaji(text: str, allowed: str = None) -> bool:
    if not text:
        return False

    augmented = allowed is not None

    if not augmented:
        return all(is_char_romaji(char) for char in text)

    regex = re.compile(allowed)
    return all(
        is_char_romaji(char) or (regex.search(char) is not None) for char in text
    )


def is_char_romaji(char: str) -> bool:
    if not char:
        return False

    return any(is_char_in_range(char, start, end) for (start, end) in ROMAJI_RANGES)



from wanakana.hiragana import is_hiragana
from wanakana.kanji import is_kanji
from wanakana.katakana import is_katakana
from wanakana.romaji import is_romaji


def is_mixed(text: str, pass_kanji=True) -> bool:
    has_kanji = False

    if not pass_kanji:
        has_kanji = any(is_kanji(char) for char in text)

    is_hiragana_or_katakana = any(is_hiragana(char) for char in text) or any(
        is_katakana(char) for char in text
    )

    return (
        is_hiragana_or_katakana
        and any(is_romaji(char) for char in text)
        and not has_kanji
    )



from wanakana.constants import HIRAGANA_START, HIRAGANA_END
from wanakana.utils import is_char_long_dash, is_char_in_range


def is_hiragana(text: str) -> bool:
    if not text:
        return False

    return all(is_char_hiragana(char) for char in text)


def any_hiragana(text: str) -> bool:
    if not text:
        return False

    return any(is_char_hiragana(char) for char in text)


def is_char_hiragana(char: str) -> bool:
    if not char:
        return False

    if is_char_long_dash(char):
        return True

    return is_char_in_range(char, HIRAGANA_START, HIRAGANA_END)



from wanakana.hiragana import is_char_hiragana
from wanakana.katakana import is_char_katakana


def is_kana(text: str) -> bool:
    if not text:
        return False

    return all(is_char_kana(char) for char in text)


def any_kana(text: str) -> bool:
    if not text:
        return False

    return any(is_char_kana(char) for char in text)


def is_char_kana(char: str) -> bool:
    if not char:
        return False

    return is_char_hiragana(char) or is_char_katakana(char)



LATIN_LOWERCASE_START = 0x61
LATIN_LOWERCASE_END = 0x7a
LATIN_UPPERCASE_START = 0x41
LATIN_UPPERCASE_END = 0x5a
LOWERCASE_ZENKAKU_START = 0xff41
LOWERCASE_ZENKAKU_END = 0xff5a
UPPERCASE_ZENKAKU_START = 0xff21
UPPERCASE_ZENKAKU_END = 0xff3a
HIRAGANA_START = 0x3041
HIRAGANA_END = 0x3096
KATAKANA_START = 0x30a1
KATAKANA_END = 0x30fc
KANJI_START = 0x4e00
KANJI_END = 0x9faf
PROLONGED_SOUND_MARK = 0x30fc
KANA_SLASH_DOT = 0x30fb

ZENKAKU_NUMBERS = [0xff10, 0xff19]
ZENKAKU_UPPERCASE = [UPPERCASE_ZENKAKU_START, UPPERCASE_ZENKAKU_END]
ZENKAKU_LOWERCASE = [LOWERCASE_ZENKAKU_START, LOWERCASE_ZENKAKU_END]
ZENKAKU_PUNCTUATION_1 = [0xff01, 0xff0f]
ZENKAKU_PUNCTUATION_2 = [0xff1a, 0xff1f]
ZENKAKU_PUNCTUATION_3 = [0xff3b, 0xff3f]
ZENKAKU_PUNCTUATION_4 = [0xff5b, 0xff60]
ZENKAKU_SYMBOLS_CURRENCY = [0xffe0, 0xffee]
HIRAGANA_CHARS = [0x3040, 0x309f]
KATAKANA_CHARS = [0x30a0, 0x30ff]
HANKAKU_KATAKANA = [0xff66, 0xff9f]
KATAKANA_PUNCTUATION = [0x30fb, 0x30fc]
KANA_PUNCTUATION = [0xff61, 0xff65]
CJK_SYMBOLS_PUNCTUATION = [0x3000, 0x303f]
COMMON_CJK = [0x4e00, 0x9fff]
RARE_CJK = [0x3400, 0x4dbf]
KANA_RANGES = [HIRAGANA_CHARS, KATAKANA_CHARS, KANA_PUNCTUATION, HANKAKU_KATAKANA]


COMMON_PRIORITIES = ["news1", "ichi1", "spec1", "spec2", "gai1"]

JA_PUNCTUATION_RANGES = [
    CJK_SYMBOLS_PUNCTUATION,
    KANA_PUNCTUATION,
    KATAKANA_PUNCTUATION,
    ZENKAKU_PUNCTUATION_1,
    ZENKAKU_PUNCTUATION_2,
    ZENKAKU_PUNCTUATION_3,
    ZENKAKU_PUNCTUATION_4,
    ZENKAKU_SYMBOLS_CURRENCY,
]

# All Japanese unicode start and end ranges
# Includes kanji, kana, zenkaku latin chars, punctuation, and number ranges.
JAPANESE_RANGES = [
    *KANA_RANGES,
    *JA_PUNCTUATION_RANGES,
    ZENKAKU_UPPERCASE,
    ZENKAKU_LOWERCASE,
    ZENKAKU_NUMBERS,
    COMMON_CJK,
    RARE_CJK,
]

MODERN_ENGLISH = [0x0000, 0x007f]
HEPBURN_MACRON_RANGES = [
    [0x0100, 0x0101],  # Ā ā
    [0x0112, 0x0113],  # Ē ē
    [0x012a, 0x012b],  # Ī ī
    [0x014c, 0x014d],  # Ō ō
    [0x016a, 0x016b],  # Ū ū
]

SMART_QUOTE_RANGES = [[0x2018, 0x2019], [0x201c, 0x201d]]  # ‘ ’  # “ ”

ROMAJI_RANGES = [MODERN_ENGLISH, *HEPBURN_MACRON_RANGES]

EN_PUNCTUATION_RANGES = [
    [0x20, 0x2f],
    [0x3a, 0x3f],
    [0x5b, 0x60],
    [0x7b, 0x7e],
    *SMART_QUOTE_RANGES,
]



from wanakana.constants import KATAKANA_START, KATAKANA_END
from wanakana.utils import is_char_in_range


def is_katakana(text: str) -> bool:
    if not text:
        return False

    return all(is_char_katakana(char) for char in text)


def any_katakana(text: str) -> bool:
    if not text:
        return False

    return any(is_char_katakana(char) for char in text)


def is_char_katakana(char: str) -> bool:
    if not char:
        return False

    return is_char_in_range(char, KATAKANA_START, KATAKANA_END)



import setuptools

with open("README.md", "r", encoding="utf8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="wanakana",
    version="1.0.1",
    author="Luke Casey",
    author_email="lc94dev@gmail.com",
    description="A library to assist in detecting Japanese text",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/luke-c/WanaKanaPython",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Natural Language :: Japanese",
    ),
)



from wanakana.constants import KANJI_START, KANJI_END
from wanakana.utils import is_char_in_range


def is_kanji(text: str) -> bool:
    if not text:
        return False

    return all(is_char_kanji(char) for char in text)


def any_kanji(text: str) -> bool:
    if not text:
        return False

    return any(is_char_kanji(char) for char in text)


def is_char_kanji(char: str) -> bool:
    if not char:
        return False

    return is_char_in_range(char, KANJI_START, KANJI_END)
